# ##### BEGIN GPL LICENSE BLOCK #####
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2
#  of the License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software Foundation,
#  Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# ##### END GPL LICENSE BLOCK #####

bl_info = {
	"name": "Blendgraph",
	"author": "Atom",
	"version": (0, 0, 1),
	"blender": (2, 6, 8), 
	"location": "Nodes > CustomNodesTree > Blendgraph",
	"description": "Parametric node-based animation",
	"warning": "User must ADD CUSTOM TREE in Composite window. Use SHIFT-A for adding new nodes.",
	"wiki_url": "",
	"tracker_url": "",
	"category": "Node"}

modules = ("utility", "ap_node_socket_integer", "ap_node_integer", "ap_node_socket_float", "ap_node_float", "ap_node_tree")
if "bpy" in locals():
	import imp
	for mod in modules:
		exec("imp.reload(%s)" % mod)
else:
	for mod in modules:
		exec("from . import %s" % mod)

import bpy, nodeitems_utils
from bpy.types import Operator, AddonPreferences
from nodeitems_utils import NodeCategory, NodeItem

def updateBlendgraphPreferences(self,context):
	util.SHOW_MESSAGES = self.show_debug
	
class BlendgraphPreferences(AddonPreferences):
	# This must match the addon name, use '__package__'
	# when defining this in a submodule of a python package.
	bl_idname = __name__

	show_debug = bpy.props.BoolProperty(name="Show Blendgraph Debug Messages In The Console", description="When enabled debug messages will appear in the console window.", default=False, options={'ANIMATABLE'}, subtype='NONE', update=updateBlendgraphPreferences)

	def draw(self, context):
		self.layout.prop(self, "show_debug")

### Node Categories ###
# Node categories are a python system for automatically
# extending the Add menu, toolbar panels and search operator.
# For more examples see release/scripts/startup/nodeitems_builtins.py

import nodeitems_utils
from nodeitems_utils import NodeCategory

# our own base class with an appropriate poll function,
# so the categories only show up in our own tree type
class bgNodeCategory(NodeCategory):
    @classmethod
    def poll(cls, context):
        return context.space_data.tree_type == 'BlendgraphTreeType'

def register():
	# Register sockets, nodes then the tree.
	ap_node_socket_float.register()
	ap_node_float.register()
	ap_node_socket_integer.register()
	ap_node_integer.register()
	ap_node_tree.register()
		
	# This registers a node with the menu system..?
	node_categories = [
		bgNodeCategory("apNode", "Numbers", items=[
			# Our basic nodes.
			NodeItem("ap_IntegerNode", label="Integer"),
			NodeItem("ap_FloatNode", label="Float"),
			]),
	]
	nodeitems_utils.register_node_categories("CUSTOM_NODES", node_categories)
	
	# Make sure the intial preferences value gets set.
	bpy.utils.register_class(BlendgraphPreferences)
	user_preferences = bpy.context.user_preferences
	addon_prefs = user_preferences.addons["blendgraph"].preferences		#key/id is name of AddOn folder (is case sensitive).
	util.SHOW_MESSAGES = addon_prefs.show_debug

def unregister():
	bpy.utils.unregister_class(BlendgraphPreferences)
	nodeitems_utils.unregister_node_categories("CUSTOM_NODES")
	
	ap_node_tree.unregister()
	ap_node_integer.unregister()
	ap_node_socket_integer.unregister()
	ap_node_float.unregister()
	ap_node_socket_float.unregister()

