# ap_node_socket_integer.
# Atom's socket definition.
# (c) 2013 

import bpy
import threading, time
from bpy.props import IntProperty, FloatProperty, StringProperty, FloatVectorProperty, CollectionProperty, EnumProperty
from bpy.types import NodeTree, Node, NodeSocket

from .utility import COLOR_LINK_MULT
from .utility import COLOR_UNLINK_MULT
from .utility import COLOR_INT

############################################################################
# Thread processing for parameters that are invalid to set in a DRAW context.
# By performing those operations in these threads we can get around the lock within a draw event.
# This is fairly abusive to the system and may result in instability of operation.
# Then again as long as you trap for stale data it just might work..?
# For best result keep the sleep time as quick as possible...no long delays here.
############################################################################
def updateNodeSocketValue(lock, passedSelf, passedValue, passedSleepTime):
	time.sleep(passedSleepTime) # Feel free to alter time in seconds as needed. 
	if passedSelf.value != passedValue:
		# New value needs to be assigned.
		passedSelf.value = passedValue

############################################################################
# Update change in my value.
############################################################################		
def updateIntegerParameter(self,context):
	try:
		scene = context.scene
	except:
		scene = None
	if scene != None:
		# Not really used right now.
		print(self.value)
		#print(dir(self))
		
# Custom socket type
class apIntegerSocket(NodeSocket):
	'''Custom node socket type'''			# Description string
	bl_idname = 'apIntegerSocketType'		# Optional identifier string. If not explicitly defined, the python class name is used.
	bl_label = 'Integer Socket'				# Label for nice name display

	value = bpy.props.IntProperty(name="Integer", description="A positive or negative number without a mantissa.", default=1, update=updateIntegerParameter)

	# Optional function for drawing the socket input value
	def draw(self, context, layout, node, text):
		if self.is_linked:
			try:
				v = self.node.inputs[text].links[0].from_socket.node.inputs[0].value
				result = str(int(v))	
				
				# We want to execute this next line right now but Blender has a lock in place.
				#self.value = v
				# Launch a delayed thread to set the value that would generate an error if issued now.
				lock = threading.Lock()
				lock_holder = threading.Thread(target=updateNodeSocketValue, args=(lock,self,v,0.02), name='Update_Node_Socket_Value')
				lock_holder.setDaemon(True)
				lock_holder.start()
			except:
				result = text
		
			layout.label(result)
		else:
			layout.prop(self, "value", text=text)

	# Socket color
	def draw_color(self, context, node):
		color_process = COLOR_INT
		if self.is_linked:
			# Socket full color when linked.
			color_result = (color_process[0]*COLOR_LINK_MULT,color_process[1]*COLOR_LINK_MULT,color_process[2]*COLOR_LINK_MULT,color_process[3]*COLOR_LINK_MULT)
		else:
			# Socket dimmer when not linked.
			color_result = (color_process[0]*COLOR_UNLINK_MULT,color_process[1]*COLOR_UNLINK_MULT,color_process[2]*COLOR_UNLINK_MULT,color_process[3]*COLOR_UNLINK_MULT)
		return color_result

def register():
	bpy.utils.register_class(apIntegerSocket)

def unregister():
	bpy.utils.unregister_class(apIntegerSocket)

if __name__ == "__main__":
	register()